@@ -14,6 +14,12 @@ mod asm_parser {
 }
 use asm_parser::{AsmParser, Rule};
 
+/// Convert one RISC-V Vector Extension(RVV) instruction into code.
+///
+/// This function try to parse the instruction as normal asm instruction or
+/// standalone label ("1:"/"label:"). If it's a valid RVV instruction, it will
+/// try to encode it to target code. Otherwise, if it's a valid asm instruction
+/// it will return `Ok(None)`. If parse or encode failed, it will return error.
 pub fn encode(inst: &str, reserved_only: bool) -> Result<Option<u32>, Error> {
     let pairs = if let Ok(result) = AsmParser::parse(Rule::inst, inst.trim()) {
         result
@@ -24,7 +30,7 @@ pub fn encode(inst: &str, reserved_only: bool) -> Result<Option<u32>, Error> {
     };
     let mut name = "unknown";
     let mut args = Vec::new();
-    for pair in pairs {
+    for pair in pairs.clone() {
         for inner1_pair in pair.into_inner() {
             match inner1_pair.as_rule() {
                 Rule::inst_name => {
@@ -33,7 +39,7 @@ pub fn encode(inst: &str, reserved_only: bool) -> Result<Option<u32>, Error> {
                 Rule::inst_arg => {
                     for inner2_pair in inner1_pair.into_inner() {
                         match inner2_pair.as_rule() {
-                            Rule::inst_arg_simple | Rule::integer => {
+                            Rule::inst_arg_mask | Rule::inst_arg_simple | Rule::integer => {
                                 args.push(inner2_pair.as_str());
                             }
                             _ => {
@@ -83,17 +89,30 @@ fn gen_inst_code(
     mut base: u32,
     arg_cfg: &[(&str, usize)],
 ) -> Result<u32, Error> {
-    let has_vm = arg_cfg.iter().any(|(arg_name, _)| *arg_name == "vm");
-    let has_nf = arg_cfg.iter().any(|(arg_name, _)| *arg_name == "nf");
+    // The order of `simm5`/`vs1` and `vs2` in opcodes-rvv is not the same with v-spec.adoc
+    let simm5_idx = arg_cfg.iter().position(|(name, _)| *name == "simm5");
+    let vs1_idx = arg_cfg.iter().position(|(name, _)| *name == "vs1");
+    let vs2_idx = arg_cfg.iter().position(|(name, _)| *name == "vs2");
+    let mut arg_cfg_vec = arg_cfg.iter().collect::<Vec<_>>();
+    if let (Some(simm5_idx), Some(vs2_idx)) = (simm5_idx, vs2_idx) {
+        arg_cfg_vec.swap(simm5_idx, vs2_idx);
+    }
+    if let (Some(vs1_idx), Some(vs2_idx)) = (vs1_idx, vs2_idx) {
+        arg_cfg_vec.swap(vs1_idx, vs2_idx);
+    }
+    let arg_cfg_final = &arg_cfg_vec;
+
+    let has_vm = arg_cfg_final.iter().any(|(arg_name, _)| *arg_name == "vm");
+    let has_nf = arg_cfg_final.iter().any(|(arg_name, _)| *arg_name == "nf");
     let number = if has_nf && has_vm {
-        arg_cfg.len() - 2
+        arg_cfg_final.len() - 2
     } else if has_vm {
-        arg_cfg.len() - 1
+        arg_cfg_final.len() - 1
     } else {
-        arg_cfg.len()
+        arg_cfg_final.len()
     };
     check_args(name, args, number, has_vm)?;
-    for (idx, (arg_name, arg_pos)) in arg_cfg.iter().rev().enumerate() {
+    for (idx, (arg_name, arg_pos)) in arg_cfg_final.iter().rev().enumerate() {
         let value = match *arg_name {
             "rs1" | "rs2" | "rd" => map_x_reg(args[idx], arg_name)?,
             "vs1" | "vs2" | "vs3" | "vd" => map_v_reg(args[idx], arg_name)?,
@@ -174,11 +193,13 @@ fn gen_inst_code(
                 }
                 value as u32
             }
+            // See comment in asm.pest
             "vm" => {
-                if args.get(idx) == Some(&"vm") {
-                    1
-                } else {
+                println!("args: {:?}", &args);
+                if args.get(idx) == Some(&"v0.t") {
                     0
+                } else {
+                    1
                 }
             }
             // FIXME: support segment load/store
@@ -301,42 +322,22 @@ mod tests {
     use super::*;
 
     #[test]
-    fn test_vsetvl() {
-        assert_eq!(
-            encode("vsetvl x5, s3, t6", false).unwrap(),
-            Some(0b10000001111110011111001011010111)
-        );
-        assert_eq!(
-            encode("vsetvli x5, s3, e8", false).unwrap(),
-            Some(0b00000000000010011111001011010111)
-        );
-        assert_eq!(
-            encode("vsetvli x5, s3, e256, m4", false).unwrap(),
-            Some(0b00000010101010011111001011010111)
-        );
-        assert_eq!(
-            encode("vsetvli x5, s3, e256, m4, ta, ma", false).unwrap(),
-            Some(0b00001110101010011111001011010111)
-        );
-        assert_eq!(
-            encode("vsetivli x5, 19, e256, m4", false).unwrap(),
-            Some(0b11000010101010011111001011010111)
-        );
-    }
-
-    #[test]
-    fn test_vle_n_v() {
-        assert_eq!(
-            encode("vle64.v v3, (a0), vm", false).unwrap(),
-            Some(0b00000010000001010111000110000111)
-        );
-    }
-
-    #[test]
-    fn test_vse_n_v() {
-        assert_eq!(
-            encode("vse64.v v3, (a0), vm", false).unwrap(),
-            Some(0b00000010000001010111000110100111)
-        );
+    fn test_insts() {
+        #[allow(clippy::deprecated_cfg_attr)]
+        #[cfg_attr(rustfmt, rustfmt_skip)]
+        for (code, inst) in [
+            (0b10000001111110011111001011010111, "vsetvl x5, s3, t6"),
+            (0b00000000000010011111001011010111, "vsetvli x5, s3, e8"),
+            (0b00000010101010011111001011010111, "vsetvli x5, s3, e256, m4"),
+            (0b00001110101010011111001011010111, "vsetvli x5, s3, e256, m4, ta, ma"),
+            (0b11000010101010011111001011010111, "vsetivli x5, 19, e256, m4"),
+            (0b00000000000001010111000110000111, "vle64.v v3, (a0), v0.t"),
+            (0b00000000000001010111000110100111, "vse64.v v3, (a0), v0.t"),
+            (0b00000010000000001000000101010111, "vadd.vv v2, v0, v1"),
+            (0b00000010000000011011000011010111, "vadd.vi v1, v0, 3"),
+            (0b00000000000000001011000011010111, "vadd.vi v1, v0, 1, v0.t"),
+        ] {
+            assert_eq!(encode(inst, false).unwrap(), Some(code), "{}", inst);
+        }
     }
 }
